<?php

namespace Drupal\video_toolbox;

use Drupal\file\Entity\File;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * The video Handler.
 */
class VideoHandler implements VideoHandlerInterface {

  /**
   * Save the video.
   */
  public function saveVideoInfo($description, $video, $user_id, $fid) {
    $database = \Drupal::database()->insert('video_store');
    $database->fields(
      [
        'description' => $description,
        'video_key' => $video,
        'uid' => $user_id,
        'fid' => $fid,
        'upload_date' => time(),
      ]
    )->execute();
  }

  /**
   * Get the video.
   */
  public function getVideoInfo($key) {

    $database = \Drupal::database()->select('video_store', 'vi');
    $database->condition('vi.video_key', $key);
    $results = $database->fields('vi',
      [
        'fid',
        'uid',
        'upload_date',
        'description',
      ]
      );
    $results = $results->execute()->fetchAll(\PDO::FETCH_ASSOC);

    return $results;
  }

  /**
   * Get the videos.
   */
  public function getVideosInfo() {
    $query = \Drupal::database()->select('video_store', 'vs');
    $query->join('file_managed', 'fm', 'fm.fid = vs.fid');
    $query->join('users_field_data', 'ufd', 'ufd.uid = vs.uid');
    $query->addField('ufd', 'name');
    $query->addField('vs', 'description');
    $query->addField('vs', 'video_key');
    $query->addField('fm', 'filemime');
    $query->addField('fm', 'filename');
    $query->addField('vs', 'upload_date');
    $results = $query->execute()->fetchAll(\PDO::FETCH_ASSOC);
    return $results;
  }

  /**
   * Update metadata.
   */
  public function updateVideos($description, $fid) {
    $query = \Drupal::database()->update('video_store');
    $query->condition('fid', [(string) $fid]);
    $query->fields(['description' => (string) $description]);
    $query->execute();

  }

  /**
   * Download a specific video with given identifier.
   *
   * @param string||int $identifier
   *   Video identifier (fid or video_id).
   * @param string $type
   *   Specifies the given identifier for the video. Possible identifiers are:
   *    - **"fid"** (File ID), internal ID used by File object.
   *    - **"video_id"** (Video ID), the random ID (Key) of each video.
   *
   * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
   *   HTTP Response.
   */
  public function downloadVideo($identifier, $type = 'fid') {

    // In case a "video_id" is sent, use it to find the needed "fid".
    if ($type === 'video_id') {
      $video = $this->getVideoInfo($identifier);
      if ($video == []) {
        return new RedirectResponse('/');
      }
      $identifier = $video[0]['fid'];
    }
    $file = File::load($identifier);
    // Get needed file information.
    $fileName = $file->getFilename();
    $fileUri = $file->getFileUri();
    // Sets HTTP response for a file.
    $baseFileName = $this->getBaseFileName($fileName);
    $response = new BinaryFileResponse($fileUri);
    $response->setContentDisposition('attachment', $baseFileName);
    /** @var \Symfony\Component\HttpFoundation\BinaryFileResponse $response */
    return $response;
  }

  /**
   * Remove unnecessary parts of the file name (Like the ID).
   *
   * @param string $fileName
   *   The file name to be sanitized.
   *
   * @return string
   *   Sanitized file name.
   */
  public function getBaseFileName($fileName) {
    // Find basic string information.
    $extPosition = strrpos($fileName, '.');
    $nameLength = strlen($fileName);
    $extLength = -1 * abs($nameLength - $extPosition);

    // Obtain versions of the string.
    $nameNoExt = substr($fileName, 0, $extPosition);
    $extension = substr($fileName, $extLength, $extPosition);

    // Remove internal ID.
    $idPosition = strrpos($nameNoExt, '_');

    // Validate if it has an ID or not. And return the base name accordingly.
    if ($idPosition === FALSE) {
      return $nameNoExt . $extension;
    }
    $basicName = substr($nameNoExt, 0, $idPosition);

    // Return the base fileName with the extension.
    $fileName = $basicName . $extension;
    return $fileName;
  }

}
